package SelectBoxBulkSetter::ContentFieldType::SelectBoxBulkSetter;
use strict;
use warnings;

sub _hdlr_custom_select {
    my ( $ctx, $args, $cond ) = @_;

    my $plugin = MT->component("SelectBoxBulkSetter");
    my $blog = MT->instance->blog;
    my $blog_id = $blog ? $blog->id : 0;
    my $select_data = $plugin->get_config_value('select_data', 'blog:' . $blog_id);

#    my @list;
#    my @select_data_split = split( /\n/, $select_data );
#    for my $row ( @select_data_split ) {
#       my @col = split( /,/, $row );
#       my $checked;
#       if ( $col[0] == 1 ) {
#           $checked = '"checked":"1"';
#       } else {
#           $checked = '"checked":""';
#       }
#       my $label   = '"label":"' . $col[1] . '"';
#       my $value   = '"value":"' . $col[2] . '"';
#       next if $label eq '' && $value eq '';
#       push @list, $checked . ',' . $label . ',' . $value;
#    }
#    $select_data = 'this.values = [{ ' . join( '},{', @list ) . ' }]';

    $select_data = 'this.values = ' . $select_data;

    return $select_data;
}

sub data_load_handler_multiple {
    my ( $app, $field_data ) = @_;
    my $field_id = $field_data->{id};
    [ $app->multi_param("content-field-${field_id}") ];
}

sub field_html_params {
    my ( $app, $field_data ) = @_;
    my $value = $field_data->{value};

    my %values;
    if ( defined $value ) {
        if ( ref $value eq 'ARRAY' ) {
            %values = map { $_ => 1 } @$value;
        }
        else {
            $values{$value} = 1;
        }
    }

    my $options = $field_data->{options};
    my $options_values = $options->{values} || [];

    @$options_values = map {
        {   l => $_->{label},
            v => $_->{value},
            $values{ $_->{value} }
            ? ( selected => 'selected="selected"' )
            : (),
        }
    } @$options_values;

    my $required = $options->{required} ? 'required' : '';

    my $multiple       = '';
    my $multiple_class = '';
    if ( $options->{multiple} ) {
        my $max = $options->{max};
        my $min = $options->{min};
        $multiple = 'multiple';
        $multiple .= qq{ data-mt-max-select="${max}"} if $max;
        $multiple .= qq{ data-mt-min-select="${min}"} if $min;
        $multiple_class = 'multiple-select';
    }

    {   multiple       => $multiple,
        multiple_class => $multiple_class,
        options_values => $options_values,
        required       => $required,
    };
}

sub field_value_handler {
    my ( $ctx, $args, $cond, $field_data, $value ) = @_;
    return $value;
}

sub html_multiple {
    my $prop = shift;
    my ( $content_data, $app, $opts ) = @_;

    my $content_field
        = MT->model('content_field')->load( $prop->content_field_id );
    my %label_hash = map { $_->{value} => $_->{label} }
        @{ $content_field->options->{values} };

    my $values = $content_data->data->{ $prop->content_field_id } || [];
    $values = [$values] unless ref $values eq 'ARRAY';

    my $can_double_encode = 1;
    my @labels
        = map { MT::Util::encode_html( $label_hash{$_}, $can_double_encode ) }
        @{$values};

    join ', ', @labels;
}

sub tag_handler_multiple {
    my ( $ctx, $args, $cond, $field_data, $value ) = @_;
    my $tok     = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    my $vars    = $ctx->{__stash}{vars} ||= {};
    my $out     = '';
    my $i       = 1;
    my $glue    = $args->{glue};

    my @values;
    if ( defined $value ) {
        if ( ref $value eq 'ARRAY' ) {
            @values = @$value;
        }
        else {
            @values = ($value);
        }
    }

    my %value_label_hash = map { $_->{value} => $_->{label} }
        @{ $field_data->{options}{values} || [] };

    for my $v (@values) {
        local $vars->{__first__}   = $i == 1;
        local $vars->{__last__}    = $i == scalar @values;
        local $vars->{__odd__}     = ( $i % 2 ) == 1;
        local $vars->{__even__}    = ( $i % 2 ) == 0;
        local $vars->{__counter__} = $i;
        local $vars->{__key__}     = $value_label_hash{$v};
        local $vars->{__value__}   = $v;

        defined(
            my $res = $builder->build(
                $ctx, $tok,
                {   %{$cond},
                    ContentFieldHeader => $i == 1,
                    ContentFieldFooter => $i == scalar @values,
                }
            )
        ) or return $ctx->error( $builder->errstr );

        if ( $res ne '' ) {
            $out .= $glue
                if defined $glue && $i > 1 && length($out) && length($res);
            $out .= $res;
            $i++;
        }
    }

    $out;
}

sub feed_value_handler_multiple {
    my ( $app, $field_data, $values ) = @_;
    unless ( ref $values eq 'ARRAY' ) {
        $values = [$values];
    }
    my %value_label_hash = map { $_->{value} => $_->{label} }
        @{ $field_data->{options}{values} || [] };

    my $contents = '';
    for my $v (@$values) {
        next unless defined $v && $v ne '';
        my $encoded_v     = MT::Util::encode_html($v);
        my $encoded_label = MT::Util::encode_html( $value_label_hash{$v} );
        $contents .= "<li>$encoded_label($encoded_v)</li>";
    }
    return "<ul>$contents</ul>";
}

sub preview_handler_multiple {
    my ( $field_data, $values, $content_data ) = @_;
    return '' unless $values;
    unless ( ref $values eq 'ARRAY' ) {
        $values = [$values];
    }
    return '' unless @$values;

    my %value_label_hash = map { $_->{value} => $_->{label} }
        @{ $field_data->{options}{values} || [] };

    my $contents = '';
    for my $v (@$values) {
        my $encoded_v     = MT::Util::encode_html($v);
        my $encoded_label = MT::Util::encode_html( $value_label_hash{$v} );
        $contents .= "<li>$encoded_label ($encoded_v)</li>";
    }
    return qq{<ul class="list-unstyled">$contents</ul>};
}

sub search_handler_multiple {
    my ( $search_regex, $field_data, $values, $content_data ) = @_;
    $values = ''        unless defined $values;
    $values = [$values] unless ref $values eq 'ARRAY';
    my %value_label_hash = map { $_->{value} => $_->{label} }
        @{ $field_data->{options}{values} || [] };
    for my $value (@$values) {
        $value = '' unless defined $value;
        my $label = $value_label_hash{$value};
        $label = '' unless defined $label;
        return 1 if $value =~ /$search_regex/ || $label =~ /$search_regex/;
    }
    0;
}

sub single_select_options_multiple {
    my $prop = shift;
    my $app  = shift || MT->app;

    my $content_field_id = $prop->{content_field_id};
    my $content_field = MT->model('content_field')->load($content_field_id);
    return $content_field->options->{values} || [];
}

sub terms_multiple {
    my $prop = shift;
    my ( $args, $base_terms, $base_args, $opts ) = @_;

    my $val       = $args->{value};
    my $data_type = $prop->{data_type};

    my $join_terms = { "value_${data_type}" => $val };
    my $cd_ids = get_cd_ids_by_inner_join( $prop, $join_terms, undef, @_ );

    if ( $args->{option} && $args->{option} eq 'is_not_selected' ) {
        $cd_ids ? { id => { not => $cd_ids } } : ();
    }
    else {
        { id => $cd_ids };
    }
}

sub tag_handler {
    my ( $ctx, $args, $cond, $field_data, $value ) = @_;
    my $tok     = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    my $vars    = $ctx->{__stash}{vars} ||= {};
    my $out     = '';
    my $i       = 1;
    my $glue    = $args->{glue};

    for my $v ( @{$value} ) {
        local $vars->{__first__}   = $i == 1;
        local $vars->{__last__}    = $i == scalar @{$value};
        local $vars->{__odd__}     = ( $i % 2 ) == 1;
        local $vars->{__even__}    = ( $i % 2 ) == 0;
        local $vars->{__counter__} = $i;
        local $vars->{__value__}   = $v;

        defined(
            my $res = $builder->build(
                $ctx, $tok,
                {   %{$cond},
                    ContentFieldHeader => $i == 1,
                    ContentFieldFooter => $i == scalar @$value,
                }
            )
        ) or return $ctx->error( $builder->errstr );

        if ( $res ne '' ) {
            $out .= $glue
                if defined $glue && $i > 1 && length($out) && length($res);
            $out .= $res;
            $i++;
        }
    }

    $out;
}

1;
